---------------------
PatchSet 3137 
Date: 2005/04/22 15:49:52
Author: cladisch
Branch: HEAD
Tag: (none) 
Log:
Summary: usb-audio - add selector unit names override for Audigy 2 NX

Add a mechanism to specify source names of selector units,
and add such names for the SB Audigy 2 NX.

Members: 
	usb/usbmixer.c:1.37->1.38 
	usb/usbmixer_maps.c:1.10->1.11 

Index: /sound/usb/usbmixer.c
diff -u /sound/usb/usbmixer.c.old /sound/usb/usbmixer.c
--- /sound/usb/usbmixer.c.old	Tue Mar 22 00:50:57 2005
+++ /sound/usb/usbmixer.c	Fri Apr 22 07:49:52 2005
@@ -70,6 +70,7 @@
 	DECLARE_BITMAP(unitbitmap, 32*32);
 	usb_audio_term_t oterm;
 	const struct usbmix_name_map *map;
+	const struct usbmix_selector_map *selector_map;
 };
 
 struct usb_mixer_elem_info {
@@ -187,6 +188,21 @@
 	return 0;
 }
 
+/* get the mapped selector source name */
+static int check_mapped_selector_name(mixer_build_t *state, int unitid,
+				      int index, char *buf, int buflen)
+{
+	const struct usbmix_selector_map *p;
+
+	if (! state->selector_map)
+		return 0;
+	for (p = state->selector_map; p->id; p++) {
+		if (p->id == unitid && index < p->count)
+			return strlcpy(buf, p->names[index], buflen);
+	}
+	return 0;
+}
+
 /*
  * find an audio control unit with the given unit id
  */
@@ -1415,7 +1431,9 @@
 			kfree(cval);
 			return -ENOMEM;
 		}
-		if (check_input_term(state, desc[5 + i], &iterm) >= 0)
+		len = check_mapped_selector_name(state, unitid, i, namelist[i],
+						 MAX_ITEM_NAME_LEN);
+		if (! len && check_input_term(state, desc[5 + i], &iterm) >= 0)
 			len = get_term_name(state, &iterm, namelist[i], MAX_ITEM_NAME_LEN, 0);
 		if (! len)
 			sprintf(namelist[i], "Input %d", i);
@@ -1521,6 +1539,7 @@
 	for (map = usbmix_ctl_maps; map->vendor; map++) {
 		if (map->vendor == state.vendor && map->product == state.product) {
 			state.map = map->map;
+			state.selector_map = map->selector_map;
 			chip->ignore_ctl_error = map->ignore_ctl_error;
 			break;
 		}
Index: /sound/usb/usbmixer_maps.c
diff -u /sound/usb/usbmixer_maps.c.old /sound/usb/usbmixer_maps.c
--- /sound/usb/usbmixer_maps.c.old	Fri Apr 22 00:52:03 2005
+++ /sound/usb/usbmixer_maps.c	Fri Apr 22 07:49:53 2005
@@ -26,10 +26,17 @@
 	int control;
 };
 
+struct usbmix_selector_map {
+	int id;
+	int count;
+	const char **names;
+};
+
 struct usbmix_ctl_map {
 	int vendor;
 	int product;
 	const struct usbmix_name_map *map;
+	const struct usbmix_selector_map *selector_map;
 	int ignore_ctl_error;
 };
 
@@ -162,6 +169,25 @@
 	{ 0 } /* terminator */
 };
 
+static struct usbmix_selector_map audigy2nx_selectors[] = {
+	{
+		.id = 14, /* Capture Source */
+		.count = 3,
+		.names = (const char*[]) {"Line", "Digital In", "What-U-Hear"}
+	},
+	{
+		.id = 29, /* Digital Out Source */
+		.count = 3,
+		.names = (const char*[]) {"Front", "PCM", "Digital In"}
+	},
+	{
+		.id = 31, /* Headphone Source */
+		.count = 2,
+		.names = (const char*[]) {"Front", "Side"}
+	},
+	{ 0 } /* terminator */
+};
+
 /* LineX FM Transmitter entry - needed to bypass controls bug */
 static struct usbmix_name_map linex_map[] = {
 	/* 1: IT pcm */
@@ -198,11 +224,29 @@
  */
 
 static struct usbmix_ctl_map usbmix_ctl_maps[] = {
-	{ 0x41e, 0x3000, extigy_map, 1 },
-	{ 0x41e, 0x3010, mp3plus_map, 0 },
-	{ 0x41e, 0x3020, audigy2nx_map, 0 },
-	{ 0x8bb, 0x2702, linex_map, 1 },
-	{ 0xc45, 0x1158, justlink_map, 0 },
+	{
+		.vendor = 0x41e, .product = 0x3000,
+		.map = extigy_map,
+		.ignore_ctl_error = 1,
+	},
+	{
+		.vendor = 0x41e, .product = 0x3010,
+		.map = mp3plus_map,
+	},
+	{
+		.vendor = 0x41e, .product = 0x3020,
+		.map = audigy2nx_map,
+		.selector_map = audigy2nx_selectors,
+	},
+	{
+		.vendor = 0x8bb, .product = 0x2702,
+		.map = linex_map,
+		.ignore_ctl_error = 1,
+	},
+	{
+		.vendor = 0xc45, .product = 0x1158,
+		.map = justlink_map,
+	},
 	{ 0 } /* terminator */
 };
 
